#include <ctime>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <opencv2/opencv.hpp>
#include <string.h>
#include "flir_scale.h"
#include "SPI.h"

using namespace cv;
using namespace std;

#define PACKET_SIZE 164
#define PACKET_SIZE_UINT16 (PACKET_SIZE/2)
#define PACKETS_PER_FRAME 60
#define FRAME_SIZE_UINT16 (PACKET_SIZE_UINT16*PACKETS_PER_FRAME)

int main()
{
	// Open camera port
	//Mats, images and resize dimensions
	Mat template_image, scale_bar, lepton_image, camera_image, overlayed_image;
	scale_bar = imread("graph.png");	//Read the bar image with color grading
	template_image = imread("template.jpg");	//The template.jpg is a simple white image 80x60 to define the dimension of matrix template_image; other ways didn't worked for me
	Size lepton_resize(640, 480);	//We define a new size for sensor image
	VideoCapture cam(0);	//Open the camera port 0 (main camera) for open cv

	//Strings
	char displayMaxValue[20];
	char displayMinValue[20];
	char displayCenterValue[20];

	//Uints
	uint8_t result[PACKET_SIZE*PACKETS_PER_FRAME];
  	uint16_t *frameBuffer;

	//Create a window
	namedWindow("Flir Lepton", 1);

	//Floats
	float minTemperature = 0;
	float maxTemperature = 0;
	float centerTemperature = 0;	

	//Open the SPI Port
	SpiOpenPort(0);
	
	while(true) {
			//Collect the data from the Flir Lepton sensor
			
			//Read the data
			int resets = 0;
			for(int j=0;j<PACKETS_PER_FRAME;j++) {

				read(spi_cs0_fd, result+sizeof(uint8_t)*PACKET_SIZE*j, sizeof(uint8_t)*PACKET_SIZE);
				int packetNumber = result[j*PACKET_SIZE+1];
				if(packetNumber != j) {
					j = -1;
					resets += 1;
					usleep(1000);
					if(resets == 750) {
						SpiClosePort(0);
						usleep(750000);
						SpiOpenPort(0);
					}
				}
			}

			frameBuffer = (uint16_t *)result;
			int row, column;
			uint16_t value;
			uint16_t minValue = 65535;
			uint16_t maxValue = 0;
			uint16_t centerValue = 0;

		
			for(int i=0;i<FRAME_SIZE_UINT16;i++) {
				if(i % PACKET_SIZE_UINT16 < 2) {
					continue;
				}
				
				//Flip the MSB and LSB at the last second
				int temp = result[i*2];
				result[i*2] = result[i*2+1];
				result[i*2+1] = temp;
				
				value = frameBuffer[i];
				if(value > maxValue) {
					maxValue = value;
				}
				if(value < minValue) {
					minValue = value;
				}
				column = i % PACKET_SIZE_UINT16 - 2;
				row = i / PACKET_SIZE_UINT16 ;
				if (column == 40 && row == 30) {
					centerValue = value;
				}
			}
	
			//Define the scale
			float diff = maxValue - minValue;
			float scale = 255/diff;
			for(int i=0;i<FRAME_SIZE_UINT16;i++) {
					if(i % PACKET_SIZE_UINT16 < 2) {
						continue;
					}
			value = (frameBuffer[i] - minValue) * scale;
			
			
			const int *colormap = flir_scale;
			column = (i % PACKET_SIZE_UINT16 ) - 2;
			row = i / PACKET_SIZE_UINT16;

			//Fill the template from before with colors
			template_image.at<Vec3b>(Point(column, row))[2] = colormap[3*value+2];
			template_image.at<Vec3b>(Point(column, row))[1] = colormap[3*value+1];
			template_image.at<Vec3b>(Point(column, row))[0] = colormap[3*value];

		}

		//Output the image
			cam >> camera_image;
			
			//Resize the 80x60 image to 640x480
			resize(template_image, lepton_image, lepton_resize);
			
			//We draw the Image
			scale_bar.copyTo(lepton_image(Rect(615, 40, 22, 400)));

			//I tried to read some temperatures from it, but is not so linear how I thought.
			//To use the sensor as a thermometer you should take the raw data and to compare it to a read temperature from a thermometer
			maxTemperature = ((maxValue / 256) * 1.40625) - 10;
			minTemperature = ((minValue / 256) * 1.40625) - 10;
			centerTemperature = ((minValue / 256) * 1.40625) - 10;
			
			//Convert the floats with 1 digit to string X.X *C
			snprintf(displayMaxValue, sizeof(displayMaxValue), "%.1f", maxTemperature);
			snprintf(displayMinValue, sizeof(displayMinValue), "%.1f", minTemperature);
			snprintf(displayCenterValue, sizeof(displayCenterValue), "%.1f", centerTemperature);
			
			
			//Overlat the images
			addWeighted(lepton_image, 0.98, captured_image, 0.02, 0.0, overlayed_image);
			
			//Display the temperatures on the screen
			putText(overlayed_image, displayMaxValue, Point(10, 40), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
			putText(overlayed_image, displayMinValue, Point(10, 80), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
			putText(overlayed_image, displayCenterValue, Point(10, 120), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
			
			//Show the image
			imshow("Flir Lepton", overlayed_image);
			
			//If a key is pressed close the program
			if(waitKey(30) >= 0) break;
	}
	
	//Close the SPI Port
	SpiClosePort(0); 
}
